part of '../../simple_type.dart';

extension ColumnExtension on Column {
  Column copyWith({
    MainAxisAlignment mainAxisAlignment,
    CrossAxisAlignment crossAxisAlignment,
    VerticalDirection verticalDirection,
    MainAxisSize mainAxisSize,
  }) {
    return Column(
      key: this.key,
      mainAxisAlignment: mainAxisAlignment ?? this.mainAxisAlignment,
      mainAxisSize: mainAxisSize ?? this.mainAxisSize,
      crossAxisAlignment: crossAxisAlignment ?? this.crossAxisAlignment,
      verticalDirection: verticalDirection ?? this.verticalDirection,
      textDirection: this.textDirection,
      textBaseline: this.textBaseline,
      children: this.children,
    );
  }

  Column get verticalDown =>
      copyWith(verticalDirection: VerticalDirection.down);

  Column get verticalUp => copyWith(verticalDirection: VerticalDirection.up);

  Column get max => copyWith(mainAxisSize: MainAxisSize.max);

  Column get min => copyWith(mainAxisSize: MainAxisSize.min);

  Column get end => copyWith(mainAxisAlignment: MainAxisAlignment.end);

  Column get mCenter => copyWith(mainAxisAlignment: MainAxisAlignment.center);

  Column get mSpaceAround =>
      copyWith(mainAxisAlignment: MainAxisAlignment.spaceAround);

  Column get mSpaceBetween =>
      copyWith(mainAxisAlignment: MainAxisAlignment.spaceBetween);

  Column get mSpaceEvenly =>
      copyWith(mainAxisAlignment: MainAxisAlignment.spaceEvenly);

  Column get mStart => copyWith(mainAxisAlignment: MainAxisAlignment.start);

  Column get cStart => copyWith(crossAxisAlignment: CrossAxisAlignment.start);

  Column get cBaseline =>
      copyWith(crossAxisAlignment: CrossAxisAlignment.baseline);

  Column get cCenter => copyWith(crossAxisAlignment: CrossAxisAlignment.center);

  Column get cEnd => copyWith(crossAxisAlignment: CrossAxisAlignment.end);

  Column get cStretch =>
      copyWith(crossAxisAlignment: CrossAxisAlignment.stretch);
}

extension RowExtension on Row {
  Row copyWith({
    MainAxisAlignment mainAxisAlignment,
    CrossAxisAlignment crossAxisAlignment,
    VerticalDirection verticalDirection,
    MainAxisSize mainAxisSize,
  }) {
    return Row(
      key: this.key,
      mainAxisAlignment: mainAxisAlignment ?? this.mainAxisAlignment,
      mainAxisSize: mainAxisSize ?? this.mainAxisSize,
      crossAxisAlignment: crossAxisAlignment ?? this.crossAxisAlignment,
      verticalDirection: verticalDirection ?? this.verticalDirection,
      textDirection: this.textDirection,
      textBaseline: this.textBaseline,
      children: this.children,
    );
  }

  Row get verticalDown => copyWith(verticalDirection: VerticalDirection.down);

  Row get verticalUp => copyWith(verticalDirection: VerticalDirection.up);

  Row get max => copyWith(mainAxisSize: MainAxisSize.max);

  Row get min => copyWith(mainAxisSize: MainAxisSize.min);

  Row get end => copyWith(mainAxisAlignment: MainAxisAlignment.end);

  Row get mCenter => copyWith(mainAxisAlignment: MainAxisAlignment.center);

  Row get mSpaceAround =>
      copyWith(mainAxisAlignment: MainAxisAlignment.spaceAround);

  Row get mSpaceBetween =>
      copyWith(mainAxisAlignment: MainAxisAlignment.spaceBetween);

  Row get mSpaceEvenly =>
      copyWith(mainAxisAlignment: MainAxisAlignment.spaceEvenly);

  Row get mStart => copyWith(mainAxisAlignment: MainAxisAlignment.start);

  Row get cStart => copyWith(crossAxisAlignment: CrossAxisAlignment.start);

  Row get cBaseline =>
      copyWith(crossAxisAlignment: CrossAxisAlignment.baseline);

  Row get cCenter => copyWith(crossAxisAlignment: CrossAxisAlignment.center);

  Row get cEnd => copyWith(crossAxisAlignment: CrossAxisAlignment.end);

  Row get cStretch => copyWith(crossAxisAlignment: CrossAxisAlignment.stretch);
}

extension WrapExtension on Wrap {
  Wrap copyWith({
    WrapCrossAlignment crossAxisAlignment,
    WrapAlignment alignment,
    WrapAlignment runAlignment,
    VerticalDirection verticalDirection,
    Axis direction,
    double spacing,
    double runSpacing,
  }) {
    return Wrap(
      children: this.children,
      clipBehavior: this.clipBehavior,
      textDirection: this.textDirection,
      key: this.key,
      runSpacing: runSpacing ?? this.runSpacing,
      spacing: spacing ?? this.spacing,
      runAlignment: runAlignment ?? this.runAlignment,
      direction: direction ?? this.direction,
      crossAxisAlignment: crossAxisAlignment ?? this.crossAxisAlignment,
      alignment: alignment ?? this.alignment,
      verticalDirection: verticalDirection ?? this.verticalDirection,
    );
  }

  Wrap spacing(double value, [bool sc = true]) =>
      copyWith(spacing: St._apply(value, sc));

  Wrap runSpacing(double value, [bool sc = true]) =>
      copyWith(runSpacing: St._apply(value, sc));

  Wrap get verticalDown => copyWith(verticalDirection: VerticalDirection.down);

  Wrap get verticalUp => copyWith(verticalDirection: VerticalDirection.up);

  Wrap get aCenter => copyWith(alignment: WrapAlignment.center);

  Wrap get aEnd => copyWith(alignment: WrapAlignment.end);

  Wrap get aSpaceAround => copyWith(alignment: WrapAlignment.spaceAround);

  Wrap get aSpaceBetween => copyWith(alignment: WrapAlignment.spaceBetween);

  Wrap get aSpaceEvenly => copyWith(alignment: WrapAlignment.spaceEvenly);

  Wrap get aStart => copyWith(alignment: WrapAlignment.start);

  Wrap get raEnd => copyWith(runAlignment: WrapAlignment.end);

  Wrap get raSpaceAround => copyWith(runAlignment: WrapAlignment.spaceAround);

  Wrap get raSpaceBetween => copyWith(runAlignment: WrapAlignment.spaceBetween);

  Wrap get raSpaceEvenly => copyWith(runAlignment: WrapAlignment.spaceEvenly);

  Wrap get raStart => copyWith(runAlignment: WrapAlignment.start);

  Wrap get dVertical => copyWith(direction: Axis.vertical);

  Wrap get dHorizontal => copyWith(direction: Axis.horizontal);

  Wrap get cStart => copyWith(crossAxisAlignment: WrapCrossAlignment.start);

  Wrap get cCenter => copyWith(crossAxisAlignment: WrapCrossAlignment.center);

  Wrap get cEnd => copyWith(crossAxisAlignment: WrapCrossAlignment.end);
}

extension StackExtension on Stack {
  Stack copyWith({
    AlignmentGeometry alignment,
    TextDirection textDirection,
    StackFit fit,
    Clip clipBehavior,
    List<Widget> children,
  }) {
    return Stack(
      key: this.key,
      textDirection: textDirection ?? this.textDirection,
      children: children ?? this.children,
      clipBehavior: clipBehavior ?? this.clipBehavior,
      fit: fit ?? this.fit,
      alignment: alignment ?? this.alignment,
    );
  }

  Stack get visible => copyWith(clipBehavior: Clip.none);

  Stack get sBottomCenter => copyWith(alignment: Alignment.bottomCenter);

  Stack get sBottomLeft => copyWith(alignment: Alignment.bottomLeft);

  Stack get sBottomRight => copyWith(alignment: Alignment.bottomRight);

  Stack get sCenter => copyWith(alignment: Alignment.center);

  Stack get sCenterLeft => copyWith(alignment: Alignment.centerLeft);

  Stack get sCenterRight => copyWith(alignment: Alignment.centerRight);

  Stack get sTopCenter => copyWith(alignment: Alignment.topCenter);

  Stack get sTopLeft => copyWith(alignment: Alignment.topLeft);

  Stack get sTopRight => copyWith(alignment: Alignment.topRight);

  Stack sAlignment(AlignmentGeometry alignment) =>
      copyWith(alignment: alignment);
}

extension ListViewExtension on ListView {
  ListView copyWith({
    ScrollPhysics physics,
    bool reverse,
    Axis scrollDirection,
    DragStartBehavior dragStartBehavior,
    EdgeInsetsGeometry padding,
  }) {
    SliverChildListDelegate delegate =
        (childrenDelegate as SliverChildListDelegate);
    return ListView(
      addAutomaticKeepAlives: delegate.addAutomaticKeepAlives,
      addRepaintBoundaries: delegate.addRepaintBoundaries,
      addSemanticIndexes: delegate.addSemanticIndexes,
      children: delegate.children,
      cacheExtent: this.cacheExtent,
      itemExtent: this.itemExtent,
      semanticChildCount: this.semanticChildCount,
      shrinkWrap: this.shrinkWrap,
      keyboardDismissBehavior: this.keyboardDismissBehavior,
      clipBehavior: this.clipBehavior,
      controller: this.controller,
      key: this.key,
      primary: this.primary,
      restorationId: this.restorationId,
      dragStartBehavior: dragStartBehavior ?? this.dragStartBehavior,
      physics: physics ?? this.physics,
      padding: padding ?? this.padding,
      reverse: reverse ?? this.reverse,
      scrollDirection: scrollDirection ?? this.scrollDirection,
    );
  }

  ListView get pClamping => copyWith(physics: const ClampingScrollPhysics());

  ListView get pBouncing => copyWith(physics: const BouncingScrollPhysics());

  ListView get pPage => copyWith(physics: const PageScrollPhysics());

  ListView get pNeverScrollable =>
      copyWith(physics: const NeverScrollableScrollPhysics());

  ListView get reverse => copyWith(reverse: !(this.reverse ?? false));

  ListView get horizontal => copyWith(scrollDirection: Axis.horizontal);

  ListView get vertical => copyWith(scrollDirection: Axis.vertical);

  ListView get dragDown => copyWith(dragStartBehavior: DragStartBehavior.down);

  ListView get dragStart =>
      copyWith(dragStartBehavior: DragStartBehavior.start);

  //! Paddings

  ListView pLTRB(double left, double top, double right, double bottom,
          [bool sc = true]) =>
      copyWith(
        padding: EdgeInsets.fromLTRB(
          St._apply(left, sc),
          St._apply(top, sc),
          St._apply(right, sc),
          St._apply(bottom, sc),
        ),
      );

  ListView p(double all, [bool sc = true]) =>
      copyWith(padding: EdgeInsets.all(St._apply(all, sc)));

  ListView pVertical(double vertical, [bool sc = true]) => copyWith(
      padding: EdgeInsets.symmetric(vertical: St._apply(vertical, sc)));

  ListView pHorizontal(double horizontal, [bool sc = true]) => copyWith(
      padding: EdgeInsets.symmetric(horizontal: St._apply(horizontal, sc)));

  ListView pSymmetric({double v, double h, bool sc = true}) => copyWith(
      padding: EdgeInsets.symmetric(
          horizontal: St._apply(h ?? 0.0, sc),
          vertical: St._apply(v ?? 0.0, sc)));

  ListView pOnly(
          {double top,
          double bottom,
          double left,
          double right,
          bool sc = true}) =>
      copyWith(
        padding: EdgeInsets.only(
          top: St._apply(top ?? 0.0, sc),
          bottom: St._apply(bottom ?? 0.0, sc),
          left: St._apply(left ?? 0.0, sc),
          right: St._apply(right ?? 0.0, sc),
        ),
      );
}

extension GridViewExtension on GridView {
  GridView copyWith({
    ScrollPhysics physics,
    bool reverse,
    Axis scrollDirection,
    DragStartBehavior dragStartBehavior,
    EdgeInsetsGeometry padding,
  }) {
    SliverChildListDelegate delegate =
        (childrenDelegate as SliverChildListDelegate);
    return GridView(
      addAutomaticKeepAlives: delegate.addAutomaticKeepAlives,
      addRepaintBoundaries: delegate.addRepaintBoundaries,
      addSemanticIndexes: delegate.addSemanticIndexes,
      children: delegate.children,
      cacheExtent: this.cacheExtent,
      gridDelegate: this.gridDelegate,
      semanticChildCount: this.semanticChildCount,
      shrinkWrap: this.shrinkWrap,
      keyboardDismissBehavior: this.keyboardDismissBehavior,
      clipBehavior: this.clipBehavior,
      controller: this.controller,
      key: this.key,
      primary: this.primary,
      restorationId: this.restorationId,
      dragStartBehavior: dragStartBehavior ?? this.dragStartBehavior,
      physics: physics ?? this.physics,
      padding: padding ?? this.padding,
      reverse: reverse ?? this.reverse,
      scrollDirection: scrollDirection ?? this.scrollDirection,
    );
  }

  GridView get pClamping => copyWith(physics: const ClampingScrollPhysics());

  GridView get pBouncing => copyWith(physics: const BouncingScrollPhysics());

  GridView get pPage => copyWith(physics: const PageScrollPhysics());

  GridView get pNeverScrollable =>
      copyWith(physics: const NeverScrollableScrollPhysics());

  GridView get reverse => copyWith(reverse: !(this.reverse ?? false));

  GridView get horizontal => copyWith(scrollDirection: Axis.horizontal);

  GridView get vertical => copyWith(scrollDirection: Axis.vertical);

  GridView get dragDown => copyWith(dragStartBehavior: DragStartBehavior.down);

  GridView get dragStart =>
      copyWith(dragStartBehavior: DragStartBehavior.start);

  //! Paddings

  GridView pLTRB(double left, double top, double right, double bottom,
          [bool sc = true]) =>
      copyWith(
        padding: EdgeInsets.fromLTRB(
          St._apply(left, sc),
          St._apply(top, sc),
          St._apply(right, sc),
          St._apply(bottom, sc),
        ),
      );

  GridView p(double all, [bool sc = true]) =>
      copyWith(padding: EdgeInsets.all(St._apply(all, sc)));

  GridView pVertical(double vertical, [bool sc = true]) => copyWith(
      padding: EdgeInsets.symmetric(vertical: St._apply(vertical, sc)));

  GridView pHorizontal(double horizontal, [bool sc = true]) => copyWith(
      padding: EdgeInsets.symmetric(horizontal: St._apply(horizontal, sc)));

  GridView pSymmetric({double v, double h, bool sc = true}) => copyWith(
      padding: EdgeInsets.symmetric(
          horizontal: St._apply(h ?? 0.0, sc),
          vertical: St._apply(v ?? 0.0, sc)));

  GridView pOnly(
          {double top,
          double bottom,
          double left,
          double right,
          bool sc = true}) =>
      copyWith(
        padding: EdgeInsets.only(
          top: St._apply(top ?? 0.0, sc),
          bottom: St._apply(bottom ?? 0.0, sc),
          left: St._apply(left ?? 0.0, sc),
          right: St._apply(right ?? 0.0, sc),
        ),
      );
}

extension SingleChildScrollViewExtension on SingleChildScrollView {
  SingleChildScrollView copyWith({
    ScrollPhysics physics,
    bool reverse,
    Axis scrollDirection,
    DragStartBehavior dragStartBehavior,
    EdgeInsetsGeometry padding,
  }) =>
      SingleChildScrollView(
        child: this.child,
        clipBehavior: this.clipBehavior,
        controller: this.controller,
        dragStartBehavior: dragStartBehavior ?? this.dragStartBehavior,
        key: this.key,
        physics: physics ?? this.physics,
        padding: padding ?? this.padding,
        primary: this.primary,
        restorationId: this.restorationId,
        reverse: reverse ?? this.reverse,
        scrollDirection: scrollDirection ?? this.scrollDirection,
      );

  SingleChildScrollView get pClamping =>
      copyWith(physics: const ClampingScrollPhysics());

  SingleChildScrollView get pBouncing =>
      copyWith(physics: const BouncingScrollPhysics());

  SingleChildScrollView get pPage =>
      copyWith(physics: const PageScrollPhysics());

  SingleChildScrollView get pNeverScrollable =>
      copyWith(physics: const NeverScrollableScrollPhysics());

  SingleChildScrollView get reverse =>
      copyWith(reverse: !(this.reverse ?? false));

  SingleChildScrollView get horizontal =>
      copyWith(scrollDirection: Axis.horizontal);

  SingleChildScrollView get vertical =>
      copyWith(scrollDirection: Axis.vertical);

  SingleChildScrollView get dragDown =>
      copyWith(dragStartBehavior: DragStartBehavior.down);

  SingleChildScrollView get dragStart =>
      copyWith(dragStartBehavior: DragStartBehavior.start);

  //! Paddings

  SingleChildScrollView pLTRB(
          double left, double top, double right, double bottom,
          [bool sc = true]) =>
      copyWith(
        padding: EdgeInsets.fromLTRB(
          St._apply(left, sc),
          St._apply(top, sc),
          St._apply(right, sc),
          St._apply(bottom, sc),
        ),
      );

  SingleChildScrollView p(double all, [bool sc = true]) =>
      copyWith(padding: EdgeInsets.all(St._apply(all, sc)));

  SingleChildScrollView pVertical(double vertical, [bool sc = true]) =>
      copyWith(
          padding: EdgeInsets.symmetric(vertical: St._apply(vertical, sc)));

  SingleChildScrollView pHorizontal(double horizontal, [bool sc = true]) =>
      copyWith(
          padding: EdgeInsets.symmetric(horizontal: St._apply(horizontal, sc)));

  SingleChildScrollView pSymmetric({double v, double h, bool sc = true}) =>
      copyWith(
          padding: EdgeInsets.symmetric(
              horizontal: St._apply(h ?? 0.0, sc),
              vertical: St._apply(v ?? 0.0, sc)));

  SingleChildScrollView pOnly(
          {double top,
          double bottom,
          double left,
          double right,
          bool sc = true}) =>
      copyWith(
        padding: EdgeInsets.only(
          top: St._apply(top ?? 0.0, sc),
          bottom: St._apply(bottom ?? 0.0, sc),
          left: St._apply(left ?? 0.0, sc),
          right: St._apply(right ?? 0.0, sc),
        ),
      );
}
