part of '../../simple_type.dart';

extension TextExtension on Text {
  Text copyWith({
    String data,
    TextStyle style,
    StrutStyle strutStyle,
    TextAlign textAlign,
    TextDirection textDirection,
    Locale locale,
    bool softWrap,
    TextOverflow overflow,
    double textScaleFactor,
    int maxLines,
    String semanticsLabel,
    TextWidthBasis textWidthBasis,
  }) =>
      Text(
        data ?? this.data,
        style: style ?? this.style,
        strutStyle: strutStyle ?? this.strutStyle,
        textAlign: textAlign ?? this.textAlign,
        locale: locale ?? this.locale,
        maxLines: maxLines ?? this.maxLines,
        overflow: overflow ?? this.overflow,
        semanticsLabel: semanticsLabel ?? this.semanticsLabel,
        softWrap: softWrap ?? this.softWrap,
        textDirection: textDirection ?? this.textDirection,
        textScaleFactor: textScaleFactor ?? this.textScaleFactor,
        textWidthBasis: textWidthBasis ?? this.textWidthBasis,
      );

  Text textStyle(TextStyle style) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          background: style?.background,
          backgroundColor: style?.backgroundColor,
          color: style?.color,
          debugLabel: style?.debugLabel,
          decoration: style?.decoration,
          decorationColor: style?.decorationColor,
          decorationStyle: style?.decorationStyle,
          decorationThickness: style?.decorationThickness,
          fontFamily: style?.fontFamily,
          fontFamilyFallback: style?.fontFamilyFallback,
          fontFeatures: style?.fontFeatures,
          fontSize: style?.fontSize,
          fontStyle: style?.fontStyle,
          fontWeight: style?.fontWeight,
          foreground: style?.foreground,
          height: style?.height,
          inherit: style?.inherit,
          letterSpacing: style?.letterSpacing,
          locale: style?.locale,
          shadows: style?.shadows,
          textBaseline: style?.textBaseline,
          wordSpacing: style?.wordSpacing,
        ),
      );

  Text textScale(double scaleFactor) =>
      this.copyWith(textScaleFactor: scaleFactor);

  AutoSizeText expandedText({
    double max = double.maxFinite,
    double min = 5,
    double stepGranularity = 1,
    bool wrapWords = true,
    AutoSizeGroup group,
    Widget overflowReplacement,
    bool sc = true,
  }) {
    max = max.toInt().toDouble();
    max = St._apply(max, sc);
    min = St._apply(min, sc);
    final style = (this.style ?? TextStyle()).copyWith(fontSize: max);
    return AutoSizeText(
      this.data,
      //! AutoSizeText
      overflowReplacement: overflowReplacement,
      wrapWords: wrapWords,
      group: group,
      stepGranularity: stepGranularity,
      maxFontSize: _fixAutoSizeFontSize(
        fontSize: max,
        stepGranularity: stepGranularity,
      ),
      minFontSize: _fixAutoSizeFontSize(
        fontSize: min,
        stepGranularity: stepGranularity,
      ),
      textKey: this.key,
      //! Text
      style: style,
      locale: this.locale,
      strutStyle: this.strutStyle,
      textAlign: this.textAlign,
      maxLines: this.maxLines,
      overflow: this.overflow,
      semanticsLabel: this.semanticsLabel,
      softWrap: this.softWrap,
      textDirection: this.textDirection,
      textScaleFactor: this.textScaleFactor,
    );
  }

  Text get bold => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontWeight: FontWeight.bold,
        ),
      );

  Text get overline => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.overline,
        ),
      );

  Text get underline => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.underline,
        ),
      );

  Text get lineThrough => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.lineThrough,
        ),
      );

  Text combine(List<TextDecoration> decorations) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.combine(decorations),
        ),
      );

  Text get italic => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontStyle: FontStyle.italic,
        ),
      );

  Text weight(FontWeight fontWeight) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontWeight: fontWeight,
        ),
      );

  Text size(double size, [bool sc = true]) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontSize: St._apply(size, sc),
        ),
      );

  Text family(String font) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontFamily: font,
        ),
      );

  Text letterSpacing(double space) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(letterSpacing: space),
      );

  Text wordSpacing(double space) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(wordSpacing: space),
      );
  Text lines(int lines) => this.copyWith(maxLines: lines);

  Text shadow({
    Color color = const Color(0x33000000),
    double blurRadius = 0.0,
    Offset offset = Offset.zero,
    bool sc = true,
  }) =>
      this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          shadows: [
            Shadow(
              color: color,
              blurRadius: St._apply(blurRadius, sc),
              offset:
                  Offset(St._apply(offset.dx, sc), St._apply(offset.dy, sc)),
            ),
          ],
        ),
      );

  double _elevationOpacityCurve(double x) =>
      pow(x, 1 / 16) / sqrt(pow(x, 2) + 2) + 0.2;

  Text textElevation(
    double elevation, {
    double angle = 0.0,
    Color color = const Color(0x33000000),
    double opacityRatio = 1.0,
    bool sc = true,
  }) {
    double calculatedOpacity = _elevationOpacityCurve(elevation) * opacityRatio;
    return shadow(
      color: color.withOpacity(calculatedOpacity),
      blurRadius: elevation,
      offset: Offset(sin(angle) * elevation, cos(angle) * elevation),
      sc: sc,
    );
  }

  Text color([Color color]) => this.copyWith(
        style: (this.style ?? TextStyle())
            .copyWith(color: color ?? St.randomPrimaryColor),
      );

  Widget border({
    StrokeCap cap = StrokeCap.round,
    StrokeJoin join = StrokeJoin.round,
    double width = 6.0,
    Color color = Colors.black,
    bool sc = true,
  }) =>
      BorderedText(
        strokeWidth: St._apply(width, sc),
        strokeColor: color,
        strokeCap: cap,
        strokeJoin: join,
        child: this,
      );

  Text tAlignment(TextAlign align) => this.copyWith(textAlign: align);

  Text get tCenter => this.copyWith(textAlign: TextAlign.center);

  Text get tEnd => this.copyWith(textAlign: TextAlign.end);

  Text get tJustify => this.copyWith(textAlign: TextAlign.justify);

  Text get tLeft => this.copyWith(textAlign: TextAlign.left);

  Text get tRight => this.copyWith(textAlign: TextAlign.right);

  Text get tStart => this.copyWith(textAlign: TextAlign.start);

  Text direction(TextDirection direction) =>
      this.copyWith(textDirection: direction);

  Text baseline(TextBaseline textBaseline) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          textBaseline: textBaseline,
        ),
      );

  Text widthBasis(TextWidthBasis textWidthBasis) =>
      this.copyWith(textWidthBasis: textWidthBasis);
}

extension AutoSizeTextExtension on AutoSizeText {
  AutoSizeText copyWith({
    double maxFontSize,
    double minFontSize,
    double stepGranularity,
    bool wrapWords,
    AutoSizeGroup group,
    Widget overflowReplacement,
  }) =>
      AutoSizeText(
        this.data,
        style: this.style,
        strutStyle: this.strutStyle,
        textAlign: this.textAlign,
        locale: this.locale,
        maxLines: this.maxLines,
        overflow: this.overflow,
        semanticsLabel: this.semanticsLabel,
        softWrap: this.softWrap,
        textDirection: this.textDirection,
        textScaleFactor: this.textScaleFactor,
        textKey: this.textKey,
        overflowReplacement: overflowReplacement ?? this.overflowReplacement,
        wrapWords: wrapWords ?? this.wrapWords,
        group: group ?? this.group,
        stepGranularity: stepGranularity ?? this.stepGranularity,
        maxFontSize: _fixAutoSizeFontSize(
          fontSize: maxFontSize ?? this.maxFontSize,
          stepGranularity: stepGranularity ?? this.stepGranularity,
        ),
        minFontSize: _fixAutoSizeFontSize(
          fontSize: minFontSize ?? this.minFontSize,
          stepGranularity: stepGranularity ?? this.stepGranularity,
        ),
      );

  AutoSizeText max(double maxFontSize, [bool sc = true]) =>
      this.copyWith(maxFontSize: St._apply(maxFontSize, sc));

  AutoSizeText min(double minFontSize, [bool sc = true]) =>
      this.copyWith(minFontSize: St._apply(minFontSize, sc));
}

double _fixAutoSizeFontSize({
  @required double stepGranularity,
  @required double fontSize,
}) {
  double extra = fontSize % stepGranularity;
  return fontSize - extra;
}
