part of '../../simple_type.dart';

extension StyledTextSpan on TextSpan {
  TextSpan copyWith({
    TextStyle style,
    GestureRecognizer recognizer,
    String semanticsLabel,
  }) =>
      TextSpan(
        text: this.text,
        children: this.children,
        style: style ?? this.style,
        recognizer: recognizer ?? this.recognizer,
        semanticsLabel: semanticsLabel ?? this.semanticsLabel,
      );

  TextSpan textStyle(TextStyle style) => this.copyWith(
        style: this.style.copyWith(
              background: style?.background,
              backgroundColor: style?.backgroundColor,
              color: style?.color,
              debugLabel: style?.debugLabel,
              decoration: style?.decoration,
              decorationColor: style?.decorationColor,
              decorationStyle: style?.decorationStyle,
              decorationThickness: style?.decorationThickness,
              fontFamily: style?.fontFamily,
              fontFamilyFallback: style?.fontFamilyFallback,
              fontFeatures: style?.fontFeatures,
              fontSize: style?.fontSize,
              fontStyle: style?.fontStyle,
              fontWeight: style?.fontWeight,
              foreground: style?.foreground,
              height: style?.height,
              inherit: style?.inherit,
              letterSpacing: style?.letterSpacing,
              locale: style?.locale,
              shadows: style?.shadows,
              textBaseline: style?.textBaseline,
              wordSpacing: style?.wordSpacing,
            ),
      );
  double _elevationOpacityCurve(double x) =>
      pow(x, 1 / 16) / sqrt(pow(x, 2) + 2) + 0.2;

  TextSpan textElevation(
    double elevation, {
    double angle = 0.0,
    Color color = const Color(0x33000000),
    double opacityRatio = 1.0,
    bool sc = true,
  }) {
    double calculatedOpacity = _elevationOpacityCurve(elevation) * opacityRatio;

    return shadow(
      color: color.withOpacity(calculatedOpacity),
      blurRadius: elevation,
      offset: Offset(sin(angle) * elevation, cos(angle) * elevation),
      sc: sc,
    );
  }

  TextSpan get bold => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontWeight: FontWeight.bold,
        ),
      );

  TextSpan get overline => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.overline,
        ),
      );

  TextSpan get underline => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.underline,
        ),
      );

  TextSpan get lineThrough => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.lineThrough,
        ),
      );

  TextSpan combine(List<TextDecoration> decorations) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          decoration: TextDecoration.combine(decorations),
        ),
      );

  TextSpan get italic => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontStyle: FontStyle.italic,
        ),
      );

  TextSpan weight(FontWeight fontWeight) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontWeight: fontWeight,
        ),
      );

  TextSpan size(double size, [bool sc = true]) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontSize: St._apply(size, sc),
        ),
      );

  TextSpan family(String font) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          fontFamily: font,
        ),
      );

  TextSpan letterSpacing(double space) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(letterSpacing: space),
      );

  TextSpan wordSpacing(double space) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(wordSpacing: space),
      );

  TextSpan shadow({
    Color color = const Color(0x33000000),
    double blurRadius = 0.0,
    Offset offset = Offset.zero,
    bool sc = true,
  }) =>
      this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          shadows: [
            Shadow(
              color: color,
              blurRadius: St._apply(blurRadius, sc),
              offset:
                  Offset(St._apply(offset.dx, sc), St._apply(offset.dy, sc)),
            ),
          ],
        ),
      );

  TextSpan color(Color color) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(color: color),
      );

  TextSpan baseline(TextBaseline textBaseline) => this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(
          textBaseline: textBaseline,
        ),
      );
}
