part of '../../simple_type.dart';

typedef GestureOnTapChangeCallback = void Function(bool tapState);

extension WidgetExtension on Widget {
  //! Paddings

  Widget pLTRB(double left, double top, double right, double bottom,
          [bool sc = true]) =>
      Padding(
        padding: EdgeInsets.fromLTRB(
          St._apply(left, sc),
          St._apply(top, sc),
          St._apply(right, sc),
          St._apply(bottom, sc),
        ),
        child: this,
      );

  Widget p(double all, [bool sc = true]) =>
      Padding(padding: EdgeInsets.all(St._apply(all, sc)), child: this);

  Widget pVertical(double vertical, [bool sc = true]) =>
      Padding(padding: EdgeInsets.symmetric(vertical: St._apply(vertical, sc)));

  Widget pHorizontal(double horizontal, [bool sc = true]) => Padding(
      padding: EdgeInsets.symmetric(horizontal: St._apply(horizontal, sc)));

  Widget pSymmetric({double v, double h, bool sc = true}) => Padding(
      padding: EdgeInsets.symmetric(
          horizontal: St._apply(h ?? 0.0, sc),
          vertical: St._apply(v ?? 0.0, sc)));

  Widget pOnly(
          {double top,
          double bottom,
          double left,
          double right,
          bool sc = true}) =>
      Padding(
          padding: EdgeInsets.only(
            top: St._apply(top ?? 0.0, sc),
            bottom: St._apply(bottom ?? 0.0, sc),
            left: St._apply(left ?? 0.0, sc),
            right: St._apply(right ?? 0.0, sc),
          ),
          child: this);

  //! Others

  Widget opacity(double opacity, {bool alwaysIncludeSemantics = false}) =>
      Opacity(
        opacity: opacity,
        alwaysIncludeSemantics: alwaysIncludeSemantics,
        child: this,
      );

  Widget bgColor([Color color]) => DecoratedBox(
      child: this,
      decoration: BoxDecoration(color: color ?? St.randomPrimaryColor));

  Widget blur(double sigma, [bool sc = true]) => BackdropFilter(
      filter: ImageFilter.blur(
        sigmaX: St._apply(sigma, sc),
        sigmaY: St._apply(sigma, sc),
      ),
      child: this);

  Widget clipRRect({
    double all,
    double topLeft,
    double topRight,
    double bottomLeft,
    double bottomRight,
    CustomClipper<RRect> clipper,
    Clip clipBehavior = Clip.antiAlias,
    bool sc = true,
  }) =>
      ClipRRect(
        child: this,
        clipper: clipper ?? null,
        clipBehavior: clipBehavior ?? null,
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(St._apply(topLeft ?? all ?? 0.0, sc)),
          topRight: Radius.circular(St._apply(topRight ?? all ?? 0.0, sc)),
          bottomLeft: Radius.circular(St._apply(bottomLeft ?? all ?? 0.0, sc)),
          bottomRight:
              Radius.circular(St._apply(bottomRight ?? all ?? 0.0, sc)),
        ),
      );

  Widget clipRect({
    CustomClipper<Rect> clipper,
    Clip clipBehavior = Clip.hardEdge,
  }) =>
      ClipRect(clipper: clipper, clipBehavior: clipBehavior, child: this);

  Widget clipOval() => ClipOval(child: this);

  Widget elevation(
    double elevation, {
    BorderRadiusGeometry borderRadius = BorderRadius.zero,
    Color shadowColor = const Color(0xFF000000),
    bool sc = true,
  }) =>
      Material(
        child: this,
        color: Colors.transparent,
        elevation: St._apply(elevation, sc),
        borderRadius: borderRadius,
        shadowColor: shadowColor,
      );

  Widget constrained({
    double width,
    double height,
    double minWidth = 0.0,
    double maxWidth = double.infinity,
    double minHeight = 0.0,
    double maxHeight = double.infinity,
    bool sc = true,
  }) {
    BoxConstraints constraints = BoxConstraints(
      minWidth: St._apply(minWidth, sc),
      maxWidth: St._apply(maxWidth, sc),
      minHeight: St._apply(minHeight, sc),
      maxHeight: St._apply(maxHeight, sc),
    );
    constraints = (width != null || height != null)
        ? constraints?.tighten(
                width: St._apply(width, sc), height: St._apply(height, sc)) ??
            BoxConstraints.tightFor(
                width: St._apply(width, sc), height: St._apply(height, sc))
        : constraints;
    return ConstrainedBox(
      child: this,
      constraints: constraints,
    );
  }

  Widget w(double width, [bool sc = true]) => ConstrainedBox(
        child: this,
        constraints: BoxConstraints.tightFor(width: St._apply(width, sc)),
      );

  Widget h(double height, [bool sc = true]) => ConstrainedBox(
        child: this,
        constraints: BoxConstraints.tightFor(height: St._apply(height, sc)),
      );

  Widget sq(double size, [bool sc = true]) => ConstrainedBox(
        child: this,
        constraints: BoxConstraints.tightFor(
          width: St._apply(size, sc),
          height: St._apply(size, sc),
        ),
      );

  Widget rotate(
    double angle, {
    Offset origin,
    AlignmentGeometry alignment = Alignment.center,
    bool transformHitTests = true,
    bool sc = true,
  }) =>
      Transform.rotate(
        child: this,
        angle: angle,
        alignment: alignment,
        origin: _offset(origin, sc),
        transformHitTests: transformHitTests,
      );

  Widget scale(
    double scale, {
    Offset origin,
    AlignmentGeometry alignment = Alignment.center,
    bool transformHitTests = true,
    bool sc = true,
  }) =>
      Transform.scale(
        scale: scale,
        alignment: alignment,
        child: this,
        origin: _offset(origin, sc),
        transformHitTests: transformHitTests,
      );

  Widget translate({
    double dx,
    double dy,
    Offset offset,
    bool transformHitTests = true,
    bool sc = true,
  }) =>
      Transform.translate(
        offset: _offset(offset, sc),
        transformHitTests: transformHitTests,
        child: this,
      );

  Widget transform({
    @required Matrix4 transform,
    Offset origin,
    AlignmentGeometry alignment,
    bool transformHitTests = true,
    bool sc = true,
  }) =>
      Transform(
        transform: transform,
        alignment: alignment,
        origin: _offset(origin, sc),
        transformHitTests: transformHitTests,
        child: this,
      );

  Widget overflow({
    AlignmentGeometry alignment = Alignment.center,
    double minWidth,
    double maxWidth,
    double minHeight,
    double maxHeight,
    bool sc = true,
  }) =>
      OverflowBox(
        alignment: alignment,
        minWidth: St._apply(minWidth, sc),
        maxWidth: St._apply(minWidth, sc),
        minHeight: St._apply(minHeight, sc),
        maxHeight: St._apply(maxHeight, sc),
        child: this,
      );

  Widget scrollView({
    Axis scrollDirection = Axis.vertical,
    bool reverse = false,
    bool primary,
    ScrollPhysics physics,
    ScrollController controller,
    DragStartBehavior dragStartBehavior = DragStartBehavior.start,
  }) =>
      SingleChildScrollView(
        child: this,
        scrollDirection: scrollDirection,
        reverse: reverse,
        primary: primary,
        physics: physics,
        controller: controller,
        dragStartBehavior: dragStartBehavior,
      );

  Widget expanded([int flex = 1]) => Expanded(child: this, flex: flex);

  Widget flexible({int flex = 1, FlexFit fit = FlexFit.loose}) => Flexible(
        child: this,
        flex: flex,
        fit: fit,
      );

  Widget positioned({
    double left,
    double top,
    double right,
    double bottom,
    double width,
    double height,
    bool sc = true,
  }) =>
      Positioned(
        child: this,
        left: St._apply(left, sc),
        top: St._apply(top, sc),
        right: St._apply(right, sc),
        bottom: St._apply(bottom, sc),
        width: St._apply(width, sc),
        height: St._apply(height, sc),
      );

  Widget semanticsLabel(String label) => Semantics.fromProperties(
        properties: SemanticsProperties(label: label),
        child: this,
      );

  Widget aspectRatio({@required double aspectRatio}) => AspectRatio(
        aspectRatio: aspectRatio,
        child: this,
      );

  Widget fittedBox(
          {BoxFit fit = BoxFit.contain,
          AlignmentGeometry alignment = Alignment.center}) =>
      FittedBox(
        fit: fit,
        alignment: alignment,
        child: this,
      );

  Widget fractionallySizedBox(
          {AlignmentGeometry alignment = Alignment.center,
          double widthFactor,
          double heightFactor}) =>
      FractionallySizedBox(
        alignment: alignment,
        widthFactor: widthFactor,
        heightFactor: heightFactor,
        child: this,
      );

  Widget decoration({
    Color color,
    DecorationImage image,
    BoxBorder border,
    BorderRadius borderRadius,
    List<BoxShadow> boxShadow,
    Gradient gradient,
    BlendMode backgroundBlendMode,
    BoxShape shape = BoxShape.rectangle,
    DecorationPosition position = DecorationPosition.background,
    bool sc = true,
  }) {
    BoxDecoration decoration = BoxDecoration(
      color: color,
      image: image,
      gradient: gradient,
      backgroundBlendMode: backgroundBlendMode,
      shape: shape,
      border: _borderApply(border, sc),
      borderRadius: _borderRadiusApply(borderRadius, sc),
      boxShadow: _boxShadowApply(boxShadow, sc),
    );
    return DecoratedBox(
      child: this,
      decoration: decoration,
      position: position,
    );
  }

  Widget get box => Container(child: this ?? null);

  //! Alignments

  Widget alignment(double x, double y) =>
      Align(alignment: Alignment(x ?? 0, y ?? 0), child: this);

  Widget bottomCenter(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.bottomCenter, child: this);

  Widget bottomLeft(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.bottomLeft, child: this);

  Widget bottomRight(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.bottomRight, child: this);

  Widget centerLeft(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.centerLeft, child: this);

  Widget centerRight(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.centerRight, child: this);

  Widget topCenter(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.topCenter, child: this);

  Widget topLeft(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.topLeft, child: this);

  Widget topRight(AlignmentGeometry alignment) =>
      Align(alignment: Alignment.topRight, child: this);

  Widget center({double widthFactor, double heightFactor}) => Center(
        widthFactor: widthFactor,
        heightFactor: heightFactor,
        child: this,
      );

  Widget from({
    Duration duration = const Duration(milliseconds: 600),
    Duration delay = Duration.zero,
    Curve curve = Curves.fastOutSlowIn,
    double angle = 0,
    double scale = 1,
    double opacity = 1,
    Offset offset = const Offset(0, 0),
    AnimationTyoe type = AnimationTyoe.normal,
    bool sc = true,
  }) =>
      AnimationWidget(
        this,
        angle: angle,
        curve: curve,
        delay: delay,
        duration: duration,
        offset: _offset(offset, sc),
        opacity: opacity,
        scale: scale,
        type: type,
      );

  //! Gestures

  Widget onTap(
    void Function() onTap, {
    Key key,
    Widget Function(BuildContext context, Widget child, bool tapped) builder,
    Duration duration = const Duration(milliseconds: 800),
    Duration delay = const Duration(milliseconds: 100),
    Curve curve = Curves.easeOutCirc,
    double angle = 0,
    double scale = 1,
    double opacity = 1,
    Offset offset = const Offset(0, 0),
    Color highlightColor,
    Color splashColor,
    bool sc = true,
  }) =>
      TapAnimation(
        onTap: onTap,
        key: key,
        builder: builder,
        duration: duration,
        delay: delay,
        curve: curve,
        angle: angle,
        opacity: opacity,
        scale: scale,
        offset: _offset(offset, sc),
        splashColor: splashColor,
        highlightColor: highlightColor,
        child: this,
      );

  Widget gestures({
    GestureOnTapChangeCallback onTapChange,
    GestureTapDownCallback onTapDown,
    GestureTapUpCallback onTapUp,
    GestureTapCallback onTap,
    GestureTapCancelCallback onTapCancel,
    GestureTapDownCallback onSecondaryTapDown,
    GestureTapUpCallback onSecondaryTapUp,
    GestureTapCancelCallback onSecondaryTapCancel,
    GestureTapCallback onDoubleTap,
    GestureLongPressCallback onLongPress,
    GestureLongPressStartCallback onLongPressStart,
    GestureLongPressMoveUpdateCallback onLongPressMoveUpdate,
    GestureLongPressUpCallback onLongPressUp,
    GestureLongPressEndCallback onLongPressEnd,
    GestureDragDownCallback onVerticalDragDown,
    GestureDragStartCallback onVerticalDragStart,
    GestureDragUpdateCallback onVerticalDragUpdate,
    GestureDragEndCallback onVerticalDragEnd,
    GestureDragCancelCallback onVerticalDragCancel,
    GestureDragDownCallback onHorizontalDragDown,
    GestureDragStartCallback onHorizontalDragStart,
    GestureDragUpdateCallback onHorizontalDragUpdate,
    GestureDragEndCallback onHorizontalDragEnd,
    GestureDragCancelCallback onHorizontalDragCancel,
    GestureDragDownCallback onPanDown,
    GestureDragStartCallback onPanStart,
    GestureDragUpdateCallback onPanUpdate,
    GestureDragEndCallback onPanEnd,
    GestureDragCancelCallback onPanCancel,
    GestureScaleStartCallback onScaleStart,
    GestureScaleUpdateCallback onScaleUpdate,
    GestureScaleEndCallback onScaleEnd,
    GestureForcePressStartCallback onForcePressStart,
    GestureForcePressPeakCallback onForcePressPeak,
    GestureForcePressUpdateCallback onForcePressUpdate,
    GestureForcePressEndCallback onForcePressEnd,
    HitTestBehavior behavior,
    bool excludeFromSemantics = false,
    DragStartBehavior dragStartBehavior = DragStartBehavior.start,
  }) =>
      GestureDetector(
        onTapDown: (TapDownDetails tapDownDetails) {
          if (onTapDown != null) onTapDown(tapDownDetails);
          if (onTapChange != null) onTapChange(true);
        },
        onTapCancel: () {
          if (onTapCancel != null) onTapCancel();
          if (onTapChange != null) onTapChange(false);
        },
        onTap: () {
          if (onTap != null) onTap();
          if (onTapChange != null) onTapChange(false);
        },
        onTapUp: onTapUp,
        onDoubleTap: onDoubleTap,
        onLongPress: onLongPress,
        onLongPressStart: onLongPressStart,
        onLongPressEnd: onLongPressEnd,
        onLongPressMoveUpdate: onLongPressMoveUpdate,
        onLongPressUp: onLongPressUp,
        onVerticalDragStart: onVerticalDragStart,
        onVerticalDragEnd: onVerticalDragEnd,
        onVerticalDragDown: onVerticalDragDown,
        onVerticalDragCancel: onVerticalDragCancel,
        onVerticalDragUpdate: onVerticalDragUpdate,
        onHorizontalDragStart: onHorizontalDragStart,
        onHorizontalDragEnd: onHorizontalDragEnd,
        onHorizontalDragCancel: onHorizontalDragCancel,
        onHorizontalDragUpdate: onHorizontalDragUpdate,
        onHorizontalDragDown: onHorizontalDragDown,
        onForcePressStart: onForcePressStart,
        onForcePressEnd: onForcePressEnd,
        onForcePressPeak: onForcePressPeak,
        onForcePressUpdate: onForcePressUpdate,
        onPanStart: onPanStart,
        onPanEnd: onPanEnd,
        onPanCancel: onPanCancel,
        onPanDown: onPanDown,
        onPanUpdate: onPanUpdate,
        onScaleStart: onScaleStart,
        onScaleEnd: onScaleEnd,
        onScaleUpdate: onScaleUpdate,
        behavior: behavior,
        excludeFromSemantics: excludeFromSemantics,
        dragStartBehavior: dragStartBehavior,
        child: this,
      );

  //! Utils

  BoxBorder _borderApply(BoxBorder border, bool sc) {
    if (border is Border) {
      return Border(
        bottom: _borderSide(border.bottom, sc),
        top: _borderSide(border.top, sc),
        left: _borderSide(border.left, sc),
        right: _borderSide(border.right, sc),
      );
    } else if (border is BorderDirectional) {
      return BorderDirectional(
        bottom: _borderSide(border.bottom, sc),
        top: _borderSide(border.top, sc),
        end: _borderSide(border.end, sc),
        start: _borderSide(border.start, sc),
      );
    }
    return border;
  }

  BorderSide _borderSide(BorderSide side, bool sc) {
    return BorderSide(
      color: side.color,
      style: side.style,
      width: St._apply(side.width, sc),
    );
  }

  BorderRadiusGeometry _borderRadiusApply(
      BorderRadiusGeometry radius, bool sc) {
    if (radius is BorderRadius) {
      return BorderRadius.only(
        bottomLeft: _radius(radius.bottomLeft, sc),
        bottomRight: _radius(radius.bottomRight, sc),
        topLeft: _radius(radius.topLeft, sc),
        topRight: _radius(radius.topRight, sc),
      );
    } else if (radius is BorderRadiusDirectional) {
      return BorderRadiusDirectional.only(
        bottomEnd: _radius(radius.bottomEnd, sc),
        bottomStart: _radius(radius.bottomStart, sc),
        topEnd: _radius(radius.topEnd, sc),
        topStart: _radius(radius.topStart, sc),
      );
    }
    return radius;
  }

  Radius _radius(Radius radius, bool sc) {
    return Radius.elliptical(St._apply(radius.x, sc), St._apply(radius.y, sc));
  }

  List<BoxShadow> _boxShadowApply(List<BoxShadow> shadows, bool sc) {
    if (shadows == null) return shadows;
    final List<BoxShadow> fixedShadows = [];
    for (final shadow in shadows) {
      fixedShadows.add(BoxShadow(
        blurRadius: shadow.blurRadius,
        color: shadow.color,
        offset: _offset(shadow.offset, sc),
        spreadRadius: St._apply(shadow.spreadRadius, sc),
      ));
    }
    return fixedShadows;
  }

  Offset _offset(Offset offset, bool sc) {
    if (offset == null) return offset;
    return Offset(St._apply(offset.dx, sc), St._apply(offset.dy, sc));
  }

  Widget get noScrollGlow =>
      ScrollConfiguration(behavior: _NoScrollGlow(), child: this);
}

Widget get boxWidget => SizedBox();

class _NoScrollGlow extends ScrollBehavior {
  @override
  Widget buildViewportChrome(
      BuildContext context, Widget child, AxisDirection axisDirection) {
    return child;
  }
}
